import sys

class UnsupportedPlatform(Exception):
    def __init__(self):
        Exception.__init__(self)
    def __repr__(self):
        msg = "This platform is not supported by Bombardier: %s"
        return msg % (sys.platform)
    def __str__(self):
        return self.__repr__()

class ConfigurationException(Exception):
    def __init__(self, reason):
        Exception.__init__(self)
        self.reason = reason
    def __repr__(self):
        msg = "This system has a configuration error: %s"
        return msg % (self.reason)
    def __str__(self):
        return self.__repr__()

class InvalidConfigData(Exception):
    def __init__(self, section, t1, t2):
        Exception.__init__(self)
        self.section = section
        self.t1      = t1
        self.t2      = t2
    def __repr__(self):
        msg = "Unable to read configuration data: %s. [expected %s, got %s]"
        return msg % (self.section, self.t1, self.t2)
    def __str__(self):
        return self.__repr__()

class StatusException(Exception):
    def __init__(self, path):
        Exception.__init__(self)
        self.path = path
    def __repr__(self):
        msg = "Error in status data located at %s"
        return msg % (self.path)
    def __str__(self):
        return self.__repr__()



#!/usr/bin/env python

"Holds data that is used as constants through different parts of the code."

# Copyright (C) 2005-2010 Peter Banka, Shawn Sherwood

from _version import version_info

# ================================================== Statics

VERSION       = str("%(branch_nick)s-%(revno)d" % version_info)
LOGS_TO_KEEP  = 5
LOG_MAX_SIZE  = 1000000

HEADER_TEXT   = \
"""Bombardier-%s, Copyright (C) 2005-2010 Peter Bankaet. al \n
Bombardier comes with ABSOLUTELY NO WARRANTY; This is free software,
and you are welcome to redistribute it under the terms of the Simplified
BSD License. License terms can be found here: 
http://www.opensource.org/licenses/bsd-license.php""" % VERSION

SERVER_CONFIG_FILE = "/etc/bombardier.yml"

VALID_PACKAGE_VERSIONS = [4, 5]

# BASIC RETURN CODES
OK        = 0
FAIL      = 1
REBOOT    = 2

RETURN_DICT = {OK: 'OK', FAIL: 'FAIL', REBOOT: 'REBOOT'}
STRING_TO_RETURN_VALUE_MAPPING = {}
for item in RETURN_DICT:
    STRING_TO_RETURN_VALUE_MAPPING[RETURN_DICT[item]] = item

# TYPES OF LOG MESSAGES FROM CLIENT TO SERVER

TRACEBACK = 0
DEBUG     = 1
INFO      = 2
WARNING   = 3
ERROR     = 4
CRITICAL  = 5

LOG_LEVEL_LOOKUP = {"DEBUG": DEBUG, "INFO": INFO, "WARNING": WARNING,
                    "ERROR": ERROR, "CRITICAL": CRITICAL}

# AUTHORIZATION LEVELS WITHIN THE CNM
USER      = 0
ADMIN     = 1

# PINSHCMD TOKEN MATCHING
NO_MATCH   = "NO_MATCH"
PARTIAL    = "PARTIAL"
COMPLETE   = "COMPLETE"
INCOMPLETE = "INCOMPLETE"

# PACKAGE ACTION COMMANDS
INSTALL          = 1
UNINSTALL        = 2
VERIFY           = 3
CONFIGURE        = 8
RECONCILE        = 10
EXECUTE          = 11
FIX              = 12
PURGE            = 13
CHECK_STATUS     = 14
DRY_RUN          = 15
INIT             = 16

ACTION_LOOKUP = {'uninstall': UNINSTALL, 'configure': CONFIGURE, 
                 'install': INSTALL, 'verify': VERIFY, 
                 'reconcile': RECONCILE, 'check_status': CHECK_STATUS, 
                 'execute': EXECUTE, 'fix': FIX, 'purge': PURGE,
                 'dry_run': DRY_RUN, "init": INIT } 

ACTION_REVERSE_LOOKUP = {}
for action in ACTION_LOOKUP:
    ACTION_REVERSE_LOOKUP[ACTION_LOOKUP[action]] = action

ACTION_DICT = {UNINSTALL: '-u', CONFIGURE:'-c', INSTALL:'-i',
               VERIFY: '-v', RECONCILE: '-r', CHECK_STATUS: '-s',
               EXECUTE: '-x', FIX: '-f', PURGE: '-p',
               DRY_RUN: '-d', INIT: '-n'}

# ENCRYPTION STUFF

CENSORED = '=== CENSORED ==='

# FILE NAMES

LOG_FILE      = "bombardier.log"
CONFIG_FILE   = "config.yml"
HASH_FILE     = "configHash.yml"
PACKAGES      = "packages"
STATUS_FILE   = "status.yml"

# TERMINAL COLORS

NO_COLOR  = 'none'
DARK      = "dark"
LIGHT     = "light"

NORMAL_COLOR = {LIGHT: '[0;30m', DARK: '[1;37m'}
GOOD_COLOR = {LIGHT: '[0;32m', DARK: '[1;32m'}
WARNING_COLOR = {LIGHT: '[0;31m', DARK: '[1;31m'}
STRONG_COLOR = {LIGHT: '[0;36m', DARK: '[1;33m'}
WEAK_COLOR = {LIGHT: '[0;37m', DARK: '[0;37m'}





#!/usr/bin/env python

"common stuff that many Bombardier modules need."

# Copyright (C) 2005-2010 Peter Banka, Shawn Sherwood

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

import os, re, time, random, yaml, md5, shutil
import sys
from static_data import OK, FAIL
from static_data import PACKAGES, STATUS_FILE, SERVER_CONFIG_FILE
from Exceptions import StatusException, UnsupportedPlatform
from Exceptions import ConfigurationException

BROKEN_INSTALL   = 0
INSTALLED        = 1
BROKEN_UNINSTALL = 2
UNINSTALLED      = 3

def load_current_progress(instance_name):
    "Loads the status.yml file for this instance"
    status_path = get_progress_path(instance_name)
    try:
        raw_data = open(status_path, 'r').read()
        data = yaml.load(raw_data)
        assert type(data) == type({})
    except Exception:
        raise StatusException(status_path)
    return data

def update_dict(newdict, olddict):
    """mashes two dictionaries together
    >>> update_dict({'a': 1}, {})
    {'a': 1}
    >>> update_dict({'a': 1}, {'b':2})
    {'a': 1, 'b': 2}
    >>> update_dict({'a': 1}, {'a':2})
    {'a': 1}
    >>> update_dict({'a': [1]}, {'a':[2]})
    {'a': [2, 1]}
    """
    for key, value in newdict.iteritems():
        if type(value) == type({}) and olddict.has_key(key):
            olddict[key] = update_dict(value, olddict[key])
        elif type(value) == type(["list"]) and olddict.has_key(key):
            olddict[key] = olddict[key] + value
        else:
            olddict[key] = value
    return olddict

def integrate(data, dictionary, overwrite):
    "mashes two dictionaries together based on timestamp"
    data["timestamp"] = time.time()
    if overwrite:
        for key, value in dictionary.iteritems():
            data[key] = value
    else:
        data = update_dict(dictionary, data)
    return data

def get_tmp_path():
    "probably could be replaced by tempfile"
    alphabet = map(chr, range(97, 123))
    tmp_fn    = "tmp"
    tmp_fn   += random.choice(alphabet)
    tmp_fn   += random.choice(alphabet)
    tmp_fn   += random.choice(alphabet)
    tmp_fn   += random.choice(alphabet)
    tmp_fn   += random.choice(alphabet)
    tmp_path  = os.path.join(get_spkg_path(), tmp_fn)
    return tmp_path

def update_progress(dictionary, instance_name, overwrite=False):
    "updates package status information"
    status_path = get_progress_path(instance_name)
    tmp_path    = get_tmp_path()
    data        = load_current_progress(instance_name)
    try:
        int_data = integrate(data, dictionary, overwrite)
        yaml_string = yaml.dump(int_data, default_flow_style=False)
        file_handle = open(tmp_path, 'w')
        file_handle.write(yaml_string)
        file_handle.flush()
        file_handle.close()
        shutil.copy(tmp_path, status_path)
        os.unlink(tmp_path)
    except IOError:
        return FAIL
    except StatusException:
        return FAIL
    return OK

def get_progress_data(instance_name, strip_version_from_name = False):
    "Obtains just the installation progress information from status.yml"
    if not instance_name:
        raise ConfigurationException("Must have a valid instance_name")
    data = load_current_progress(instance_name)
    if data.has_key("install-progress"):
        progress_data = data["install-progress"]
        if strip_version_from_name:
            return strip_version_from_keys(progress_data)
        return progress_data
    return {}

def cygpath(dos_path):
    "Converts an MS-DOS path to a POSIX path"
    prefix = ''
    if not dos_path:
        return ''
    if ':' in dos_path:
        drive = dos_path.split(':')[0]
        prefix = '/cygdrive/%s' % drive
        dos_path = dos_path.split(':')[-1]
    return prefix + dos_path.replace('\\', '/')

def hash_list(listobj):
    """hashes all the values of a list

    >>> hash_list(['a', 1])
    ['0cc175b9c0f1b6a831c399e269772661', 'c4ca4238a0b923820dcc509a6f75849b']
    """
    output = []
    for value in listobj:
        if type(value) == type({}):
            output.append(hash_dictionary(value))
        elif type(value) == type([]):
            output.append(hash_list(value))
        elif type(value) == type('string'):
            output.append(md5.new(value).hexdigest())
        elif  type(value) == type(1):
            output.append(md5.new(`value`).hexdigest())
    return output

def hash_dictionary(dictionary):
    """goes through a dictionary and turns all the values in it
    to md5 hashes. This is useful for saving off the information
    about a configuration without saving the configuration itself

    >>> hash_dictionary({'a': 2, 'b': {'c': 'terrance'}})
    {'a': 'c81e728d9d4c2f636f067f89cc14862c', 'b': {'c': '370574fb6b12bfabab1b23a7b8c11c0a'}}
    """
    output = {}
    for key in dictionary.keys():
        value = dictionary[key]
        if type(value) == type({}):
            output[key] = hash_dictionary(value)
        elif type(value) == type([]):
            output[key] = hash_list(value)
        elif type(value) == type('string'):
            output[key] = md5.new(value).hexdigest()
        elif type(value) == type(1):
            output[key] = md5.new(`value`).hexdigest()
    return output

def diff_lists(sub_list, super_list, check_values=False):
    """Finds the differences between two lists


    """
    differences = []
    if len(sub_list) == 0:
        return []
    if type(sub_list[0]) == type(super_list[0]):
        if type(sub_list[0]) == type(1) or type(sub_list[0]) == type('string'):
            # comparing a list of literals, length doesn't matter.
            return []
    if len(sub_list) != len(super_list):
        differences = list(set(sub_list) - set(super_list))
    for index in range(0, len(sub_list)):
        sub_value = sub_list[index]
        super_value = super_list[index]
        if type(sub_value) != type(super_value):
            differences.append(sub_value)
            continue
        if type(sub_value) == type({}):
            differences.append(diff_dicts(sub_value, super_value, check_values))
            continue
        elif type(sub_value) == type([]):
            differences.append(diff_lists(sub_value, super_value, check_values))
            continue
        elif type(sub_value) == type('string') or type(sub_value) == type(1):
            if check_values and sub_value != super_value:
                differences.append(sub_value)
                continue
        else:
            differences.append(sub_value)
            continue
    return differences

def diff_dicts(sub_dict, super_dict, check_values=False):
    """Finds all the entries in sub_dict that are not in super_dict.
    
    >>> diff_dicts({'a':1}, {'b':2})
    {'a': 1}

    >>> diff_dicts({'a':1, 'b':2}, {'b':2})
    {'a': 1}

    >>> diff_dicts({'b': 2}, {'a':1, 'b':2})
    {}

    When lists are encountered, only the type is relevant, and not the values.
    For example:

    >>> diff_dicts({'b': [2]}, {'a':1, 'b':[1,2,3,4]})
    {}
    
    >>> diff_dicts({'b': 2}, {'a':1, 'b':[1,2,3,4]})
    {'b': 2}

    Similarly, all scalar values are treated the same, whether they are string
    or character values:
    >>> diff_dicts({'b': 2}, {'a':1, 'b':'2'})
    {}

    Unless the check_values flag is given:
    >>> diff_dicts({'b': 2}, {'a':1, 'b':'2'}, True)
    {'b': 2}
    """
    differences = {}
    for sub_key in sub_dict.keys():
        if sub_key not in super_dict.keys():
            differences[sub_key] = sub_dict[sub_key]
            continue
        sub_value = sub_dict[sub_key]
        super_value = super_dict[sub_key]
        if type(sub_value) == type('string') or type(sub_value) == type(1):
            if type(super_value) != type('string') and \
              type(super_value) != type(1):
                differences[sub_key] = sub_value
                continue
            if not check_values:
                continue
            if sub_value != super_value:
                differences[sub_key] = sub_value
                continue
            continue
        elif type(sub_value) != type(super_value):
            differences[sub_key] = sub_value
            continue
        elif type(sub_value) == type({}):
            diff = diff_dicts(sub_value, super_value, check_values)
            if diff != {}:
                differences[sub_key] = diff
            continue
        elif type(sub_value) == type([]):
            diff = diff_lists(sub_value, super_value, check_values)
            if diff != []:
                differences[sub_key] = diff
            continue
        else:
            differences[sub_key] = sub_value
            continue
    return differences

def compare_dicts(sub_dict, super_dict, check_values=False):
    "Tells you if two dictionaries are the same or not"
    if diff_dicts(sub_dict, super_dict, check_values) == {}:
        return True
    return False

def datesort(thing_one, thing_two):
    "used by sort() to sort a list"
    if type(thing_one) == type(["list"]):
        if type(thing_two) == type(["list"]):
            if thing_one[1]:
                if thing_two[1]:
                    return thing_one[1] - thing_two[1]
    return False

def get_time_struct(time_string):
    """Convert the time_string into a time struct. If a package has
    been both installed and uninstalled, it's necessary to known which
    one came more recently so that we know if the package is on the
    system or not.

    >>> get_time_struct("NA")
    0

    >>> get_time_struct("Fri Dec  4 17:13:46 2009")
    1259975626
    """
    if time_string == "NA":
        return 0
    try:
        timestruct = int(time.mktime(time.strptime(time_string)))
    except ValueError:
        timestruct = int(time.time())
    return timestruct

def strip_version(package_name):
    """removes the -\d version number from a package name
    >>> strip_version("abc")
    'abc'
    >>> strip_version("abc-2")
    'abc'
    >>> strip_version("abc-2834343-2")
    'abc-2834343'
    """

    if package_name.rfind('-') == -1:
        return package_name
    ending = package_name[package_name.rfind('-')+1:]
    validator = re.compile("([0-9]+)")
    if validator.search(ending):
        if validator.search(ending).groups()[0] == ending:
            package_name = package_name[:package_name.rfind('-')]
    return package_name

def strip_version_from_keys(progress_data):
    """removes the "-\d" version information from the progress info
    >>> b = {'LdapServer-19': {'UNINSTALLED': 'NA', 
    ...                        'VERIFIED': 'Fri Dec  4 17:13:46 2009',
    ...                        'DEPENDENCY_ERRORS': [],
    ...                        'INSTALLED': 'Fri Dec  4 17:13:46 2009'}}
    >>> strip_version_from_keys(b)
    {'LdapServer': {'UNINSTALLED': 'NA', 'VERIFIED': 'Fri Dec  4 17:13:46 2009', 'DEPENDENCY_ERRORS': [], 'INSTALLED': 'Fri Dec  4 17:13:46 2009'}}
    """
    output = {}
    for key in progress_data.keys():
        output[strip_version(key)] = progress_data[key]
    return output

def determine_install_status(item, progress_data):
    '''Goes through the progress data that is stored in status.yml,
    and determines which packages are broken-not installed, which are installed,
    which are broken-installed, and which are uninstalled.

    >>> b = {'LdapServer-19': {'UNINSTALLED': 'NA',
    ...                        'VERIFIED': 'Fri Dec  4 17:13:46 2009',
    ...                        'DEPENDENCY_ERRORS': [],
    ...                        'INSTALLED': 'Fri Dec  4 17:13:46 2009'}}
    >>> determine_install_status("LdapServer-19", b)
    (1, 1259975626)

    >>> b["LdapServer-19"]["UNINSTALLED"] = 'Sat Dec  5 17:13:46 2009'
    >>> determine_install_status("LdapServer-19", b)
    (3, 1260062026)
    '''
    if progress_data[item].get("INSTALLED") == None:
        progress_data[item]["INSTALLED"] = ''
    if progress_data[item].get("UNINSTALLED") == None:
        progress_data[item]["UNINSTALLED"] = ''
    inst_txt = progress_data[item]["INSTALLED"]
    uninst_txt = progress_data[item]["UNINSTALLED"]
    if inst_txt == "BROKEN":
        return BROKEN_INSTALL, None
    if uninst_txt == "BROKEN":
        return BROKEN_UNINSTALL, None
    if inst_txt == "NA":
        return UNINSTALLED, None
    inst_int = get_time_struct(inst_txt)
    if uninst_txt != "NA":
        uninst_int = get_time_struct(uninst_txt)
        if uninst_int > inst_int:
            return UNINSTALLED, uninst_int
    return INSTALLED, inst_int

def get_installed_uninstalled_times(progress_data):
    """Provides information about what time installation or uninstallation
    activities have taken place

    >>> b = {'LdapServer-19': {'UNINSTALLED': 'NA',
    ...                        'VERIFIED': 'Fri Dec  4 17:13:46 2009',
    ...                        'DEPENDENCY_ERRORS': [],
    ...                        'INSTALLED': 'Fri Dec  4 17:13:46 2009'}}
    >>> get_installed_uninstalled_times(b)
    {'broken_uninstalled': [], 'uninstalled': [], 'broken_installed': [], 'installed': [['LdapServer-19', 1259975626]]}
    """
    output = {"installed":[], "uninstalled":[], 
              "broken_installed":[], "broken_uninstalled":[]}
    for item in progress_data.keys():
        status, last_action = determine_install_status(item, progress_data)
        if status == INSTALLED:
            output["installed"].append([item, last_action])
        elif status == UNINSTALLED:
            output["uninstalled"].append([item, last_action])
        elif status == BROKEN_INSTALL:
            output["broken_installed"].append([item, last_action])
        elif status == BROKEN_UNINSTALL:
            output["broken_uninstalled"].append([item, last_action])
    output["installed"].sort(datesort)
    output["uninstalled"].sort(datesort)
    return output

def rpartition(string, char):
    "Python2.5's rpartition method. Useful in old versions of Python"
    chunks = string.split(char)
    if len(chunks) == 1:
        return ('', '', string)
    first = char.join(chunks[:-1])
    last = chunks[-1]
    return (first, char, last)

def check_if_more_recent(base_package_name, action_time, comparison_list):
    """If a package has both an INSTALL action and an UNINSTALL action
    shown next to it, we need to determine which is the more recent and
    then ignore the other one.
    """
    remove = False
    for full_package_name_2, action_time_2 in comparison_list:
        base_package_name_2 = rpartition(full_package_name_2, '-')[0]
        if base_package_name == base_package_name_2:
            if action_time_2 > action_time:
                remove = True
    return remove

def strip_version_info(pkg_info):
    '''
    This strips '-\d' version information from each package_name in
    the pkg_info dictionary.
    pkg_info -- a dictionary of ["installed"], ["uninstalled"],
                ["broken_installed"], ["broken_uninstalled"]. Each item
                is a tuple of package_name / date.
    '''
    output = {"installed":[], "uninstalled":[], 
              "broken_installed":[], "broken_uninstalled":[]}
    package_list_names = output.keys()

    # look at package_list_name for duplicates in other listTypes
    for package_list_name in package_list_names:
        # do other package lists have this basename?
        for full_package_name, action_time in pkg_info[package_list_name]:
            base_package_name = rpartition(full_package_name, '-')[0]
            other_types = output.keys()
            other_types.remove(package_list_name)
            for compare_list in other_types:
                more_recent = check_if_more_recent(base_package_name, action_time,
                                              pkg_info[compare_list])
                if more_recent == True:
                    break
            if not more_recent:
                output[package_list_name].append([base_package_name,
                                                  action_time])
    return output

def get_installed(progress_data):
    """Tells you what packages are installed (and which are broken)

    >>> b = {'LdapServer-19': {'UNINSTALLED': 'NA',
    ...                        'VERIFIED': 'Fri Dec  4 17:13:46 2009',
    ...                        'DEPENDENCY_ERRORS': [],
    ...                        'INSTALLED': 'Fri Dec  4 17:13:46 2009'}}
    >>> get_installed(b)
    (['LdapServer'], [])

    >>> b["LdapServer-19"]["UNINSTALLED"] = 'Fri Dec  4 20:13:46 2009'
    >>> get_installed(b)
    ([], [])

    >>> b["LdapServer-19"]["UNINSTALLED"] = 'BROKEN'
    >>> get_installed(b)
    ([], ['LdapServer'])
    """
    pkg_info = get_installed_uninstalled_times(progress_data)
    pkg_info = strip_version_info(pkg_info)
    installed_package_names = [package_name[0] for package_name in pkg_info["installed"]]
    broken_package_names    = [package_name[0] for package_name in pkg_info["broken_installed"]]
    broken_package_names   += [package_name[0] for package_name in pkg_info["broken_uninstalled"]]
    return installed_package_names, broken_package_names

# CONFIGURATION FILE METHODS

def get_linux_config():
    "our config file is in /etc/bombardier.yml. Go read it and give us the info"
    try:
        data = open(SERVER_CONFIG_FILE, 'r').read()
    except IOError, ioe:
        raise ConfigurationException(str(ioe))
    config = yaml.load(data)
    return config

def put_linux_config(config):
    "Write a change to our config file"
    data = open("/etc/bombardier.yml", 'w')
    data.write(yaml.dump(config))

def add_dictionaries(dict1, dict2):
    """dict1 gets stuff from dict2, only if it doesn't have it"""
    for key,value in dict2.iteritems():
        if not dict1.has_key(key):
            dict1[key] = value
        else:
            if type(value) == type(dict1[key]):
                if type(value) == type(dict()):
                    dict1[key] = add_dictionaries(dict1[key], value)
    return dict1

def get_spkg_path():
    "Find out where our root directory is on the client"
    spkg_path = ''
    if sys.platform == "linux2" or sys.platform == "cygwin":
        config = get_linux_config()
        spkg_path = config.get("spkg_path")
        if not spkg_path:
            spkg_path = config.get("spkgPath")
    elif sys.platform == "win32":
        import _winreg as winreg
        key_name = r"Software\GE-IT\Bombardier"
        try:
            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
                                 key_name, 0, winreg.KEY_QUERY_VALUE)
            spkg_path, dummy = winreg.QueryValueEx(key, "InstallPath")
        except:
            spkg_path = r"C:\spkg"
    else:
        raise UnsupportedPlatform()
    return spkg_path

def get_package_path(instance_name):
    "Find out where our package repository is"
    return os.path.join(get_spkg_path(), instance_name, PACKAGES)

def get_progress_path(instance_name):
    "Find out where our 'status.yml' file is"
    new_path = os.path.join(get_spkg_path(), instance_name, STATUS_FILE)
    return new_path

if __name__ == "__main__":
    import doctest
    doctest.testmod()




#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'1.00',
 'build_date': '2010-01-28 10:13:13 -0800',
 'clean': None,
 'date': '2010-01-27 17:44:36 -0800',
 'revision_id': 'root@peacemaker-20100128014436-gbd9bn8vgy2n81u5',
 'revno': 721}

revisions = {}

file_revisions = {}



if __name__ == '__main__':
    print 'revision: %(revno)d' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info



#!/cygdrive/c/Python24/python.exe

# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Copyright (C) 2005-2010 Peter Banka et al

'''Config.py: This class provides the functionality of providing
heirarchichal configuration data through either a dictionary
information to be stored either on the server or locally in the
config.yml file in the bombardier home directory.'''


import yaml
from mini_utility import hash_dictionary, diff_dicts, get_spkg_path
import copy, os
from Exceptions import InvalidConfigData
from Logger import Logger
from static_data import OK, FAIL, CONFIG_FILE

class Config(dict):
    """This object retains the complete configuration for a system.
    It tries to impersonate a dictionary as well as provide a nice
    string interface to the data it holds"""

    def __init__(self, instance_name, config_data):
        '''
        instance -- the name of this machine
        config_data -- configuration information for this machine
        '''
        dict.__init__(self)
        self.instance_name = instance_name
        self.includes   = []
        self.automated  = False
        self.username   = None
        self.password   = None
        self.domain     = None
        if not config_data:
            config_data = {}
        self.data       = config_data
        self._read_local_config()

    def _read_local_config(self):
        '''If there is a cleartext configuration file on the system,
        we will read that and ignore the configuration sent to us from
        the server.'''
        spkg_path = get_spkg_path()
        if not spkg_path:
            return FAIL
        config_path = os.path.join(get_spkg_path(), self.instance_name,
                                  CONFIG_FILE)
        if not os.path.isfile(config_path):
            return FAIL
        msg = "DETECTED A CLEARTEXT LOCAL CONFIGURATION FILE:"\
              " IGNORING MANAGEMENT SERVER CONFIGURATION"
        Logger.warning(msg)
        file_handle = open(config_path, 'r')
        try:
            config_data = file_handle.read()
            self.data = yaml.load(config_data)
        except:
            return FAIL
        return OK

    def __getitem__(self, key):
        'for dictionary impersonation'
        return self.data[key]

    def __setitem__(self, key, value):
        'for dictionary impersonation'
        self.data[key] = value

    def keys(self):
        'for dictionary impersonation'
        return self.data.keys()

    def getInstance(self):
        'deprecated function'
        Logger.warning("getInstance is Deprecated: use get_instance instead")
        return self.get_instance()

    def get_instance(self):
        'getter'
        return self.instance_name

    def set_bom_pkgs(self, bom_pkns):
        'setter'
        self.data["packages"] = copy.deepcopy(bom_pkns)

    def get_bom_pkns(self):
        'getter'
        if self.data.has_key("packages"):
            return copy.deepcopy(self.data["packages"])
        return []

    def save_hash(self, path):
        '''
        Save off a hash of our config data for future reference
        path -- the place to find the configuration hash that was used the
                last time this package was configured or installed
        '''
        try:
            file_handle = open(path, 'w')
            hash_dict = hash_dictionary(self.data)
            hash_yaml = yaml.dump(hash_dict)
            file_handle.write(hash_yaml)
            file_handle.close()
        except IOError:
            return FAIL
        return OK

    def check_hash(self, path):
        '''
        We saved off a hash of our configuration data the last time we did some
        package maintenance. Now we're going to load that saved value and
        compare it with our current config. That will tell us what values have
        changed.
        path -- the place to find the configuration hash that was used the
                last time this package was configured or installed
        '''
        old_config = {}
        difference = {}
        try:
            yaml_string = open(path, 'r').read()
            old_config = yaml.load(yaml_string)
            new_config = hash_dictionary(self.data)
            difference = diff_dicts(old_config, new_config, check_values=True)
        except IOError:
            msg = "Could not load saved configuration data in %s" % path
            Logger.warning(msg)
        except IndexError:
            msg = "Could not compare configuration data in %s" % path
            Logger.warning(msg)
        except:
            Logger.warning("Bad yaml in file %s" % path)
        return difference

    def set(self, section, option, value):
        'dictionary impersonation'
        if type(self.data.get(section)) != type(dict()):
            self.data[section] = {}
        self.data[section][option] = value
        return OK

    def has_section(self, section_query):
        'ConfigParser impersonation'
        for section in self.data.keys():
            if section.lower() == section_query.lower():
                if type(self.data[section]) == type({}):
                    return True
        return False

    def has_option(self, section_query, option_query):
        'ConfigParser impersonation'
        for section in self.data.keys():
            if section.lower() == section_query.lower():
                section_data = self.data[section]
                if type(section_data) == type({}):
                    for option in section_data.keys():
                        if option.lower() == option_query.lower():
                            return True
        return False

    def get_raw(self, section, option, default=None, optional=True):
        '''Get any object from the configuration
        section -- this is a top-level value
        option -- a key under the top-level
        default -- a default value, if not set
        optional -- whether the system should require the value to
                    be set for a machine that implements a package
        '''
        if self.data.has_key(section):
            if self.data[section].has_key(option):
                return self.data[section][option]
        for key in self.data.keys():
            if key.lower() == section.lower():
                if type(self.data[key] == type({})):
                    for subkey in self.data[key].keys():
                        if subkey.lower() == option.lower():
                            return self.data[key][subkey]
        if default != None:
            if not self.data.has_key(section):
                self.data[section] = {}
            self.data[section][option] = default
            return default
        else:
            if self.data.has_key(section):
                msg = "Option %s not found for section %s" % (option, section)
                raise InvalidConfigData(msg, None, None)
            else:
                msg = "Section %s not found" % section
                raise InvalidConfigData(msg, None, None)

    def get(self, section, option, default='', optional=True):
        'dictionary impersonation'
        return str(self.get_raw(section, option, default))

    def get_dict(self, section, option, default = {}, optional = True):
        '''Get a dictionary object from the configuration
        section -- this is a top-level value
        option -- a key under the top-level
        default -- a default value, if not set
        optional -- whether the system should require the value to
                    be set for a machine that implements a package
        '''
        result = self.get_raw(section, option, default)
        if result.__class__ == {}.__class__:
            return result
        else:
            raise TypeError

    def _parse_section(self, section_string, default, optional):
        '''Provides 'dotted-value' access to configuration data
        section_string -- a string to get configuration 
                          (e.g. 'svn.server.home_directory')
        default -- default value
        optional -- whether it should be required by machine config
        '''
        sections = section_string.split('.')
        dat = self.data
        for section in sections:
            try:
                dat = dat[section]
            except KeyError:
                dat = None
                break
        if dat == None:
            if not optional:
                msg = "Option %s not found" % section_string
                raise InvalidConfigData(msg, None, None)
            dat = default
        return dat

    def _getobj(self, section_string, default, expected_type, optional):
        '''dotted-value access to a generic object
        section_string -- configuration section
        default -- default value
        expected_type -- type we should return
        optional -- whether it must be implemented
        '''
        value = self._parse_section(section_string, default, optional)        
        if type(expected_type) == type("string"):
            if type(value) == type(1234) or type(value) == type(123.32):
                value = str(value)
        if type(value) == type(expected_type):
            return value
        raise InvalidConfigData(section_string, type(value),
                                type(expected_type))

    def listobj(self, section_string, default=[], optional=True):
        'Get a list object from the config'
        return self._getobj(section_string, default, [], optional)

    def string(self, section_string, default='', optional=True):
        '''Get a string object from the config
        >>> data = {"section": {"item1": "value1", "item2": 2, "item3": [1,2,3]}}
        >>> config = Config("tester", data)
        >>> config.string("section.item1")
        "value1"
        >>> config.string("section.item234", default = "not_defined")
        "not_defined"
        >>> config.string("section.item2")
        "2"
        '''
        return self._getobj(section_string, default, "string", optional)

    def integer(self, section_string, default=1, optional=True):
        '''Get a integer object from the config
        >>> data = {"section": {"item1": "value1", "item2": 2, "item3": [1,2,3]}}
        >>> config = Config("tester", data)
        >>> config.integer("section.item2")
        2
        '''
        return self._getobj(section_string, default, 1, optional)

    def dictionary(self, section_string, default={}, optional=True):
        '''Get a dictionary object from the config'
        >>> data = {"section": {"item1": "value1", "item2": 2, "item3": [1,2,3]}}
        >>> config = Config("tester", data)
        >>> config.dictionary("section")
        {"item1": "value1", "item2": 2, "item3": [1,2,3]}
        '''
        return self._getobj(section_string, default, {}, optional)

if __name__ == "__main__":
    import doctest
    doctest.testmod()




#!/usr/bin/env python

"sets bombardier-core up."

from distutils.core import Command, setup
from unittest import TextTestRunner, TestLoader
from glob import glob
from os.path import splitext, basename, join as pjoin, walk
import os
from lib._version import version_info

LONG_DESCRIPTION = """\
Bombardier is a software system that delivers visibility, control
and automation to datacenter environments. Bombardier provides a means
for changes to be rolled out to a network of servers in a highly
controlled way, providing optimum security, logging, centralized
change control.

What happens when systems you're responsible for go offline?

    * Do you get paged?
    * Are you part of a team that needs to respond immediately?
    * Are your products or services potentially at risk?
    * Is your job or livelihood at risk?

If the answer is yes to any of these questions, then Bombardier may be
for you. Bombardier makes application deployment, minor changes, and
routine maintenance much more reliable and secure. It also provides you
with a degree of insight into the changes that are going on within your
production datacenter that you may have never seen before.
"""


class CleanCommand(Command):
    user_options = [ ]
    def initialize_options(self):
        self._clean_me = [ ]
        for root, dirs, files in os.walk('.'):
            for f in files:
                if f.endswith('.pyc'):
                    self._clean_me.append(pjoin(root, f))

    def finalize_options(self):
        pass

    def run(self):
        for clean_me in self._clean_me:
            try:
                os.unlink(clean_me)
            except:
                pass

class TestCommand(Command):
    user_options = [ ]
    def initialize_options(self):
        self._dir = os.getcwd()

    def finalize_options(self):
        pass

    def run(self):
        '''
        Finds all the tests modules in tests/, and runs them.
        '''
        testfiles = [ ]
        for t in glob(pjoin(self._dir, 'tests', '*.py')):
            if not t.endswith('__init__.py'):
                testfiles.append('.'.join(
                    ['tests', splitext(basename(t))[0]])
                )

        tests = TestLoader().loadTestsFromNames(testfiles)
        t = TextTestRunner(verbosity = 1)
        t.run(tests)


setup (name = "bombardier_core",
       description = "Open Source Configuration management and package delivery: shared components",
       version = str("%(branch_nick)s-%(revno)d" % version_info),
       long_description = LONG_DESCRIPTION,
       packages = ["bombardier_core"],
       package_dir = {"bombardier_core": "lib"},
       author = "Peter Banka and Shawn Sherwood",
       author_email = "peter.banka@gmail.com",
       url = "http://bombardierinstaller.org/",
       install_requires = [ 'PyYAML', 'pycrypto' ],
       provides = [ 'bombardier_core' ],
       classifiers = [
           "Development Status :: 2 - Pre-Alpha",
           "Programming Language :: Python",
           "Intended Audience :: Developers",
           "Intended Audience :: Information Technology",
           "License :: OSI Approved :: BSD License",
           "Operating System :: POSIX",
           "Topic :: Software Development :: Libraries :: Python Modules",
           "Topic :: System :: Archiving :: Packaging",
       ],
       cmdclass = { 'test': TestCommand, 'clean': CleanCommand }
)
